/***************************************************************************
                          game.h  -  game class header file
                             -------------------
    begin                : Fri Oct 4 2002
    copyright            : (C) 2002 by Andrew D. Balsa
    email                : andrebalsa@mailingaddress.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GAME_H
#define GAME_H

/*--------------  Defines  ------------------*/

#define PLAY_MOVE (0)   /* code for normal play */
#define RESIGN (-2)     /* code for resigning */
#define PASS (-1)       /* pass */
#define BOTHPASS (-3)   /* code for both sides passed, game is over*/

/*codes for players; and a code for something (e.g. a point on the board) */
/* which belongs to neither*/
enum bVal {BLACK, WHITE, EMPTY};
typedef enum bVal sType;


/*--------------  Globals  ------------------*/

// Class game

class game {

public:

   int RANDOM_SEED;        /* seed for rand() Random Number Generator */
   int kox, koy;           /* coords of current ko, or negative for none */

   enum bVal pla;          /* code for next player to play */
   int tur;                /* the number of this move */

   int ex;                 /* x coordinate of last move generated by engine */
   int ey;                 /* y coordinate of last move generated by engine */

   string emove;           /* vertex in letternumber format of last move by engine */
   
   float komi;             /* komi value for this game */
   
   int blackHandicap;      /* number of handicap stones for Black */

   enum bVal amigoColor;   /* color amigo is playing */

   int qpa;                /* flag for the last move was a pass, so that if this */
                           /* move is a pass, the game is finished */

   int boardSize;          /* Size of board which we are using */
                           /* (an optional command line parameter) */
                           /* default = 9 */

   int capturedBlack;      /* Number of black stones that White has captured */

   int capturedWhite;      /* Number of white stones that Black has captured */

   int territoryBlack;     /* Estimated Black territory */

   int territoryWhite;     /* Estimated White territory */
};

#define TRUE 1
#define FALSE 0

#define MAXGROUPS 160

#define PLACED 0
#define REMOVED 1

#define numPoints thegame.boardSize
#define maxPoint numPoints - 1

#define maxSIZE 19

/*-- definitions used when counting up --*/

#define CNT_UNDECIDED	0
#define CNT_BLACK_TERR	1
#define CNT_WHITE_TERR	2
#define CNT_NOONE	3

/*-- macro functions --*/

/*Return TRUE if (x,y) corresponds to a point on the board.*/
#define onboard(x) (0<=(x)&&(x)<thegame.boardSize)
#define LegalPoint(x,y) (onboard(x)&&onboard(y))
#define ForeachPoint(a,b)	for(a=0;a<thegame.boardSize;a++) for (b=0;b<thegame.boardSize;b++)

struct Group
{
   enum bVal color; /* The color of the group */
   short code,      /* The code used to mark stones in the group */
         count,     /* The number of stones in the group */
         internal,  /* The number of internal liberties */
         external,  /* The number of external liberties */
         liberties, /* The total number of liberties */
         eyes,      /* The number of eyes */
         alive,     /* A judgement of how alive this group is */
         territory; /* The territory this group controls */
};

struct bRec
{
  enum bVal	Val;    /* What is at this intersection */
  short   xOfs,
          yOfs;
  short   mNum;
  short   GroupNum; /* What group the stone belongs to */
  short   marked;   /* TRUE or FALSE */
};

#endif // GAME_H
